@@ -16,9 +16,12 @@
 
 """Functions for working with generic syntax URIs."""
 
-import pkg_resources
-__version__ = pkg_resources.resource_string(
-    "lazr.batchnavigator", "version.txt").strip()
+try:
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    import importlib_metadata
+
+__version__ = importlib_metadata.version('lazr.batchnavigator')
 
 # While we generally frown on "*" imports, this, combined with the fact we
 # only test code from this module, means that we can verify what has been
