@@ -16,10 +16,11 @@
 
 __metaclass__ = type
 
-import urllib
-import cgi
-
-from zope.interface import implements, classProvides
+from six.moves.urllib.parse import (
+    parse_qs,
+    urlencode,
+    )
+from zope.interface import implementer, provider
 from zope.interface.common.sequence import IFiniteSequence
 from zope.cachedescriptors.property import Lazy
 
@@ -36,14 +37,13 @@ __all__ = [
     'ListRangeFactory',
     ]
 
+@implementer(IBatchNavigator)
+@provider(IBatchNavigatorFactory)
 class BatchNavigator:
 
     # subclasses can override
     _batch_factory = _Batch
 
-    implements(IBatchNavigator)
-    classProvides(IBatchNavigatorFactory)
-
     start_variable_name = 'start'
     batch_variable_name = 'batch'
     memo_variable_name = 'memo'
@@ -79,7 +79,7 @@ class BatchNavigator:
         # happen outside of tests.)
         if query_string is None:
             query_string = ''
-        return cgi.parse_qs(query_string, keep_blank_values=True)
+        return parse_qs(query_string, keep_blank_values=True)
 
     def __init__(self, results, request, start=0, size=None, callback=None,
                  transient_parameters=None, force_start=False,
@@ -244,7 +244,7 @@ class BatchNavigator:
         """
         # We need the doseq=True because some url params are for multi-value
         # fields.
-        return urllib.urlencode(self.getCleanQueryParams(params), doseq=True)
+        return urlencode(self.getCleanQueryParams(params), doseq=True)
 
     def generateBatchURL(self, batch, backwards=False):
         url = ""
@@ -270,7 +270,7 @@ class BatchNavigator:
             params.append((self.start_variable_name, start))
 
         base_url = str(self.request.URL)
-        return "%s?%s" % (base_url, urllib.urlencode(params))
+        return "%s?%s" % (base_url, urlencode(params))
 
     def firstBatchURL(self):
         batch = self.batch.firstBatch()
@@ -308,6 +308,7 @@ class BatchNavigator:
             setattr(self, attrname, prefix + getattr(self, attrname))
 
 
+@implementer(IRangeFactory)
 class ListRangeFactory:
     """Implements an IRangeFactory for lists (and list-like objects).
 
@@ -315,8 +316,6 @@ class ListRangeFactory:
     low', 'from this point and up'.
     """
 
-    implements(IRangeFactory)
-
     def __init__(self, results):
         self.results = results
 
@@ -365,9 +364,15 @@ class ListRangeFactory:
                 offset = total_length
                 # Storm raises an exception for a negative initial offset.
                 offset_plus_size = max(offset + offset_plus_size, 0)
-            result = list(self.results[offset_plus_size:offset])
-            result.reverse()
-            return result
+            if offset_plus_size == offset:
+                # Save a query if the slice is empty.
+                # offset cannot be None so we don't have to deal with the
+                # effect of None parameters as slice boundaries.
+                return []
+            else:
+                result = list(self.results[offset_plus_size:offset])
+                result.reverse()
+                return result
 
     def getSliceByIndex(self, start, end):
         """See `IRangeFactory`"""
