@@ -35,9 +35,14 @@ class ListWithExplosiveLen(list):
 
 class ListAllergicToNegativeIndexes(list):
     """A list subclass that doesn't like negative indexes."""
-    def __getitem__(self, index):
-        if index < 0:
-            raise RuntimeError('Negative indexes are not allowed.')
+    def __getitem__(self, key):
+        if isinstance(key, slice):
+            if key.start < 0 or key.stop < 0:
+                raise RuntimeError('Negative indexes are not allowed.')
+        elif isinstance(key, int):
+            if key < 0:
+                raise RuntimeError('Negative indexes are not allowed.')
+        return super(ListAllergicToNegativeIndexes, self).__getitem__(key)
 
 
 class ListWithIncorrectLength(list):
@@ -61,8 +66,15 @@ class ListWithIncorrectLength(list):
     def __len__(self):
         return self._length
 
-    def __getslice__(self, start, end):
-        return super(ListWithIncorrectLength, self).__getslice__(start, end)
+    def __getitem__(self, key):
+        # Python 3 needs a little extra help to behave in the odd way we
+        # want for tests using this class.
+        if isinstance(key, slice):
+            if key.start < 0:
+                key = slice(max(0, len(self) + key.start), key.stop, key.step)
+            if key.stop < 0:
+                key = slice(key.start, len(self) + key.stop, key.step)
+        return super(ListWithIncorrectLength, self).__getitem__(key)
 
 
 class RangeFactoryWithValueBasedEndpointMemos:
@@ -224,7 +236,7 @@ class BatchTest(unittest.TestCase):
                 'seven', 'eight', 'nine', 'ten']
 
     def test_Interface(self):
-        self.failUnless(IBatch.providedBy(self.getBatch()))
+        self.assertTrue(IBatch.providedBy(self.getBatch()))
 
     def getBatch(self, start=0, size=None, range_forwards=None,
                  range_memo=None):
@@ -309,14 +321,14 @@ class BatchTest(unittest.TestCase):
 
     def test__contains__(self):
         batch = self.getBatch(0, 3)
-        self.assert_(batch.__contains__('one'))
-        self.assert_(batch.__contains__('two'))
-        self.assert_(batch.__contains__('three'))
-        self.assert_(not batch.__contains__('four'))
+        self.assertTrue(batch.__contains__('one'))
+        self.assertTrue(batch.__contains__('two'))
+        self.assertTrue(batch.__contains__('three'))
+        self.assertFalse(batch.__contains__('four'))
         batch = self.getBatch(6, 3)
-        self.assert_(not batch.__contains__('one'))
-        self.assert_(batch.__contains__('seven'))
-        self.assert_(not batch.__contains__('ten'))
+        self.assertFalse(batch.__contains__('one'))
+        self.assertTrue(batch.__contains__('seven'))
+        self.assertFalse(batch.__contains__('ten'))
 
     def test_last_batch_len_optimization(self):
         # If the current batch is known to be the last batch, then we can
@@ -645,9 +657,8 @@ class BatchTest(unittest.TestCase):
 
     def test_PartialBatch(self):
         # PartialBatch implements the full IBatch interface.
-        from zope.interface.common.mapping import IItemMapping
         self.assertTrue(verifyClass(IBatch, _PartialBatch))
-        partial = _PartialBatch(sliced_list=range(3))
+        partial = _PartialBatch(sliced_list=list(range(3)))
         self.assertTrue(verifyObject(IBatch, partial))
         # trueSize is the length of sliced_list
         self.assertEqual(3, partial.trueSize)
@@ -722,7 +733,7 @@ class BatchTest(unittest.TestCase):
         # Batches get the length of the result set from the
         # range factory.
         range_factory = RangeFactoryRecordingResultLengthCalls()
-        batch = _Batch(range(3), range_factory)
+        batch = _Batch(list(range(3)), range_factory)
         listlength = batch.listlength
         self.assertEqual(42, listlength)
         self.assertEqual(1, range_factory.rough_length_accesses)
