@@ -20,10 +20,11 @@ Zope-derived Batching Support
 The intent is to get rid of this in favor of moving to extending z3c.batching.
 """
 
+from __future__ import absolute_import
 
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface.common.sequence import IFiniteSequence
-from interfaces import IBatch
+from .interfaces import IBatch
 
 from zope.cachedescriptors.property import Lazy
 
@@ -31,6 +32,7 @@ from zope.cachedescriptors.property import Lazy
 # as BatchNavigator. In Launchpad, we override it via a config option.
 BATCH_SIZE = 50
 
+@implementer(IBatch)
 class _PartialBatch:
     """A helper batch implementation.
 
@@ -57,7 +59,6 @@ class _PartialBatch:
     we use this helper class for the getEndpointMemos() call in
     _Batch.sliced_list().
     """
-    implements(IBatch)
 
     def __init__(self, sliced_list):
         self.start = 0
@@ -114,8 +115,8 @@ class _PartialBatch:
     has_next_batch = None
 
 
+@implementer(IBatch)
 class _Batch(object):
-    implements(IBatch)
 
     def _get_length(self, results):
         # If this batch's contents is smaller than the batch size, it is the
@@ -191,13 +192,13 @@ class _Batch(object):
 
     def __getitem__(self, key):
         if key >= self.trueSize:
-            raise IndexError, 'batch index out of range'
+            raise IndexError('batch index out of range')
         # When self.start is negative (IOW, when we are batching over an
         # empty list) we need to raise IndexError here; otherwise, the
         # attempt to slice below will, on objects which don't implement
         # __len__, raise a mysterious exception directly from python.
         if self.start < 0:
-            raise IndexError, 'batch is empty'
+            raise IndexError('batch is empty')
         # We delegate to self.sliced_list because:
         # - usually folk will be walking the batch and sliced_list optimises
         #   that.
